@@ -1,4 +1,5 @@
 import os
+from time import sleep
 import pytest
 from django.urls import reverse
 from django.contrib.auth import get_user_model
@@ -26,9 +27,11 @@ from polymorphic.tests.models import (
     Model2B,
     Model2C,
     Model2D,
+    NoChildren,
 )
 
 from playwright.sync_api import sync_playwright, expect
+from urllib.parse import urljoin
 
 
 class PolymorphicAdminTests(AdminTestCase):
@@ -169,6 +172,9 @@ class _GenericAdminFormTest(StaticLiveServerTestCase):
     admin_password = "password"
     admin = None
 
+    def admin_url(self):
+        return f"{self.live_server_url}{reverse('admin:index')}"
+
     def add_url(self, model):
         path = reverse(f"admin:{model._meta.label_lower.replace('.', '_')}_add")
         return f"{self.live_server_url}{path}"
@@ -198,22 +204,19 @@ class _GenericAdminFormTest(StaticLiveServerTestCase):
         cls.playwright = sync_playwright().start()
         cls.browser = cls.playwright.chromium.launch(headless=cls.HEADLESS)
 
-        cls.admin = get_user_model().objects.create_superuser(
-            username=cls.admin_username, email="admin@example.com", password=cls.admin_password
-        )
-
     @classmethod
     def tearDownClass(cls):
         """Clean up Playwright instance after tests."""
         cls.browser.close()
         cls.playwright.stop()
-        if cls.admin:
-            cls.admin.delete()
         super().tearDownClass()
         del os.environ["DJANGO_ALLOW_ASYNC_UNSAFE"]
 
     def setUp(self):
         """Create an admin user before running tests."""
+        self.admin = get_user_model().objects.create_superuser(
+            username=self.admin_username, email="admin@example.com", password=self.admin_password
+        )
         self.page = self.browser.new_page()
         # Log in to the Django admin
         self.page.goto(f"{self.live_server_url}/admin/login/")
@@ -230,20 +233,10 @@ class _GenericAdminFormTest(StaticLiveServerTestCase):
 
 
 class StackedInlineTests(_GenericAdminFormTest):
-    def setUp(self):
-        PlainA.objects.all().delete()
-        InlineParent.objects.all().delete()
-        super().setUp()
-        for name in ["Brian", "Alice", "Emma", "Anna"]:
-            PlainA.objects.create(field1=name)
-
-    def tearDown(self):
-        PlainA.objects.all().delete()
-        InlineParent.objects.all().delete()
-        super().tearDown()
-
     def test_admin_inline_add_autocomplete(self):
         # https://github.com/jazzband/django-polymorphic/issues/546
+        for name in ["Brian", "Alice", "Emma", "Anna"]:
+            PlainA.objects.create(field1=name)
         self.page.goto(self.add_url(InlineParent))
         self.page.fill("input[name='title']", "Parent 1")
         with self.page.expect_navigation(timeout=10000) as nav_info:
@@ -285,16 +278,73 @@ class StackedInlineTests(_GenericAdminFormTest):
         suggestions = self.page.locator("ul.select2-results__options > li").all_inner_texts()
         assert suggestions == ["Brian"]
 
+    def test_inline_form_ordering_and_removal(self):
+        """
+        Test that the javascript places the inline forms in the correct order on
+        repeated adds without a save.
+
+        https://github.com/jazzband/django-polymorphic/issues/426
+        """
+        self.page.goto(self.add_url(InlineParent))
+
+        polymorphic_menu = self.page.locator(
+            "div.polymorphic-add-choice div.polymorphic-type-menu"
+        )
+
+        self.page.click("div.polymorphic-add-choice a")
+        polymorphic_menu.wait_for(state="visible")
+        self.page.click("div.polymorphic-type-menu a[data-type='inlinemodelb']")
+        polymorphic_menu.wait_for(state="hidden")
+        self.page.click("div.polymorphic-add-choice a")
+        polymorphic_menu.wait_for(state="visible")
+        self.page.click("div.polymorphic-type-menu a[data-type='inlinemodela']")
+        polymorphic_menu.wait_for(state="hidden")
+        self.page.click("div.polymorphic-add-choice a")
+        polymorphic_menu.wait_for(state="visible")
+        self.page.click("div.polymorphic-type-menu a[data-type='inlinemodela']")
+        polymorphic_menu.wait_for(state="hidden")
+        self.page.click("div.polymorphic-add-choice a")
+        polymorphic_menu.wait_for(state="visible")
+        self.page.click("div.polymorphic-type-menu a[data-type='inlinemodelb']")
+        polymorphic_menu.wait_for(state="hidden")
+
+        inline0 = self.page.locator("div#inline_children-0")
+        inline1 = self.page.locator("div#inline_children-1")
+        inline2 = self.page.locator("div#inline_children-2")
+        inline3 = self.page.locator("div#inline_children-3")
+
+        inline0.wait_for(state="visible")
+        inline1.wait_for(state="visible")
+        inline2.wait_for(state="visible")
+        inline3.wait_for(state="visible")
+
+        assert "model b" in inline0.inner_text() and "#1" in inline0.inner_text()
+        assert "model a" in inline1.inner_text() and "#2" in inline1.inner_text()
+        assert "model a" in inline2.inner_text() and "#3" in inline2.inner_text()
+        assert "model b" in inline3.inner_text() and "#4" in inline3.inner_text()
+
+        # Now remove inline 2 and check the numbering is correct
+        inline1.locator("a.inline-deletelink").click()
+        # the ids are updated - so we expect the last div id to be removed
+        inline3.wait_for(state="detached")
+        assert "model b" in inline0.inner_text() and "#1" in inline0.inner_text()
+        assert "model a" in inline1.inner_text() and "#2" in inline1.inner_text()
+        assert "model b" in inline2.inner_text() and "#3" in inline2.inner_text()
+
+        inline0.locator("a.inline-deletelink").click()
+        inline2.wait_for(state="detached")
+        assert "model a" in inline0.inner_text() and "#1" in inline0.inner_text()
+        assert "model b" in inline1.inner_text() and "#2" in inline1.inner_text()
+
+        inline1.locator("a.inline-deletelink").click()
+        inline1.wait_for(state="detached")
+        assert "model a" in inline0.inner_text() and "#1" in inline0.inner_text()
+
+        inline0.locator("a.inline-deletelink").click()
+        inline0.wait_for(state="detached")
+
 
 class PolymorphicFormTests(_GenericAdminFormTest):
-    def setUp(self):
-        Model2A.objects.all().delete()
-        super().setUp()
-
-    def tearDown(self):
-        Model2A.objects.all().delete()
-        super().tearDown()
-
     def test_admin_polymorphic_add(self):
         model2b_ct = ContentType.objects.get_for_model(Model2B)
         model2c_ct = ContentType.objects.get_for_model(Model2C)
@@ -321,7 +371,7 @@ class PolymorphicFormTests(_GenericAdminFormTest):
                 {
                     "field1": "2D1",
                     "field2": "2D2",
-                    "field3": "2D3",
+                    # "field3": "2D3", excluded!
                     "field4": "2D4",
                 },
             ),
@@ -367,5 +417,107 @@ class PolymorphicFormTests(_GenericAdminFormTest):
 
         assert Model2D.objects.first().field1 == "2D1"
         assert Model2D.objects.first().field2 == "2D2"
-        assert Model2D.objects.first().field3 == "2D3"
+        assert Model2D.objects.first().field3 == ""
         assert Model2D.objects.first().field4 == "2D4"
+
+
+class PolymorphicNoChildrenTests(_GenericAdminFormTest):
+    def test_admin_no_polymorphic_children(self):
+        self.page.goto(self.add_url(NoChildren))
+        self.page.fill("input[name='field1']", "NoChildren1")
+        with self.page.expect_navigation(timeout=10000) as nav_info:
+            self.page.click("input[name='_save']")
+
+        response = nav_info.value
+        assert response.status < 400
+
+        # verify the add
+        added = NoChildren.objects.get(field1="NoChildren1")
+        self.page.goto(self.change_url(NoChildren, added.pk))
+        assert self.page.locator("input[name='field1']").input_value() == "NoChildren1"
+
+
+class AdminRecentActionsTests(_GenericAdminFormTest):
+    def test_admin_recent_actions(self):
+        """
+        Test that recent actions links respect polymorphism
+        """
+        model2a_ct = ContentType.objects.get_for_model(Model2A)
+        model2d_ct = ContentType.objects.get_for_model(Model2D)
+
+        for model_type, fields in [
+            (
+                model2a_ct,
+                {
+                    "field1": "2A1",
+                },
+            ),
+            (
+                model2d_ct,
+                {
+                    "field1": "2D1",
+                    "field2": "2D2",
+                    # "field3": "2D3",  excluded!
+                    "field4": "2D4",
+                },
+            ),
+        ]:
+            self.page.goto(self.add_url(Model2A))
+            self.page.locator(f"input[type=radio][value='{model_type.pk}']").check()
+            with self.page.expect_navigation(timeout=10000) as nav_info:
+                self.page.click("input[name='_save']")
+
+            response = nav_info.value
+            assert response.status < 400
+
+            for field, value in fields.items():
+                self.page.fill(f"input[name='{field}']", value)
+
+            with self.page.expect_navigation(timeout=10000) as nav_info:
+                self.page.click("input[name='_save']")
+
+            response = nav_info.value
+            assert response.status < 400
+
+        self.page.goto(self.admin_url())
+        links = self.page.locator("ul.actionlist a")
+        count = links.count()
+
+        # Collect hrefs
+        hrefs = []
+        for i in range(count):
+            href = links.nth(i).get_attribute("href")
+            if href:  # ignore missing hrefs just in case
+                hrefs.append(href)
+
+        assert hrefs, "No links found in .actionlist"
+
+        # Visit each link and ensure the HTTP status is OK
+        for href in hrefs:
+            action_url = urljoin(self.live_server_url, href)
+            response = self.page.goto(action_url)
+            assert response is not None, f"No response for {action_url}"
+            assert response.ok, f"{action_url} returned bad status {response.status}"
+            if "model2a" in action_url:
+                inputs = self.page.locator("#model2a_form input[type='text']")
+                count = inputs.count()
+                assert count == 1
+
+                values = []
+                for i in range(count):
+                    values.append(inputs.nth(i).input_value())
+
+                assert values == ["2A1"]
+            elif "model2d" in action_url:
+                # this also tests that exclusion of field3 works
+                inputs = self.page.locator("#model2d_form input[type='text']")
+                count = inputs.count()
+                assert count == 3
+
+                values = []
+                for i in range(count):
+                    values.append(inputs.nth(i).input_value())
+
+                assert values == ["2D1", "2D2", "2D4"]
+            else:
+                assert False, f"Unexpected change url: {action_url}"
